#!/usr/bin/python

# Two player game: goal is to not be the last to take any sticks.

# Computer AI Strategy: Always force the other player to choose from a number
# of sticks that is 1 more than a multiple of 4. Once we do so once, we're
# guaranteed to always be able to do so.

# This blog post was helpful: http://blogs.britannica.com/2007/05/math-and-winning-strategies/

from random import randint

def get_user_input(num_sticks):
    prompt = "How many sticks will you take (between 1 and 3)? "
    take = raw_input(prompt)
    while not take.isdigit() or int(take) <= 0 or int(take) > 3:
        print "That was not a valid number of sticks."
        take = raw_input(prompt)
    return int(take)

def computer_turn(num_sticks):
    # can take up to 3, but less if there are less than 3 sticks remaining
    max_sticks = 3 if num_sticks > 3 else num_sticks
    if num_sticks <= 2:
        return 1
    elif num_sticks == 3:
        return 2
    elif num_sticks == 4:
        return 3
    else:
        # make other player choose from 1 more than the highest multiple of 4, less than num_sticks
        # if possible
        return (num_sticks % 4) - 1 if num_sticks % 4 > 1 else randint(1,3)


def is_game_over(sticks, player):
        if len(sticks) == 0:
            print "Game Over!", player, "player took the last stick and lost."
            exit()

def game():
    # start with a random amount of sticks
    sticks = "|" * randint(5,30)
    while len(sticks) > 0:
        # Human Plays First Each Turn
        print "There are", len(sticks), "sticks remaining."
        print sticks, '\n'
        take = get_user_input(len(sticks))
        sticks = "|" * (len(sticks) - take)
        is_game_over(sticks, "Human")
        # Computer Plays Second Each Turn
        computer_take = computer_turn(len(sticks))
        print "The computer took", computer_take, "sticks."
        sticks = "|" * (len(sticks) - computer_take)
        is_game_over(sticks, "Computer")

game()
